#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;
out vec4 color;

//uniform vec3 textColor;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
//uniform sampler2D diffuseTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

//uniforms for brick texture
uniform vec3 brickColor, mortarColor;
uniform vec2 brickSize;
uniform vec2 brickPct;
//const float levels = 3.0f;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);

	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	vec2 procPosWorldXY = vert.position_world.xy;
	vec3 procColor;
	vec2, procPos, procUseBrick;


	procPos = procPosWorldXY/brickSize;

	if(fract(procPos.y*0.5f) > 0.5f) {
		procPos.x += 0.5f;
	}

	procPos = fract(procPos);
	procUseBrick = step(procPos, brickPct);	

	procColor = mix(mortarColor, brickColor, procUseBrick.x*procUseBrick.y);

	vec3 texColor = vec3(0.5f,0.25f,0.30f);
	color = vec4(procColor * materialCoefficients.x, 1); // ambient

	
	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * procColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * procColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
}

