#version 430 core

in vec3 position_world;
in vec3 normal_world;
in vec2 uv;

uniform vec3 camera_world;
uniform sampler2D diffuseTexture;
uniform vec3 materialCoefficients;

out vec4 color;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

void main() {
	vec3 view = normalize(camera_world - position_world);
	vec3 pointLightDirection = normalize(pointL.position - position_world);
	vec3 pointLightReflection = reflect(-pointLightDirection, normal_world);
	
	float ambient = materialCoefficients.x;
	float diffuseInt = materialCoefficients.y;
	float specularInt = materialCoefficients.z;
	float diffusePoint = max(dot(normalize(normal_world), pointLightDirection), 0.0) * diffuseInt;
	float specularPoint = pow(max(dot(view, pointLightReflection), 0.0), 1) * specularInt;

	vec3 ambientColorPoint = pointL.color * ambient;
	vec3 diffuseColorPoint = pointL.color * floor(diffusePoint * 4) * 1/4;
	vec3 specularColorPoint = pointL.color * specularPoint * floor(diffusePoint * 4) * 1/4;

	vec3 directionalLightReflection = reflect(-dirL.direction, normal_world);
	float diffuseDirectional = max(dot(normalize(normal_world), dirL.direction), 0.0);
	float specularDirectional = pow(max(dot(view, directionalLightReflection), 0.0), 1) * specularInt;
	
	vec3 ambientColorDir = dirL.color * ambient;
	vec3 diffuseColorDir = dirL.color * floor(diffuseDirectional * 4) * 1/4;
	vec3 specularColorDir = dirL.color * specularDirectional * floor(diffuseDirectional * 4) * 1/4;

	vec3 texture = texture(diffuseTexture, uv).rgb * (ambientColorPoint + diffuseColorPoint + specularColorPoint);
	color = vec4(texture, 1.0);
	
}